@@ -30,6 +30,7 @@ from types import FunctionType
 from inspect import getsource
 from textwrap import dedent
 import warnings
+import sys
 
 from couchdb import http, json, util
 
@@ -43,7 +44,9 @@ DEFAULT_BASE_URL = os.environ.get('COUCHDB_URL', 'http://localhost:5984/')
 class Server(object):
     """Representation of a CouchDB server.
 
-    >>> server = Server()
+    >>> server = Server() # connects to the local_server
+    >>> remote_server = Server('http://example.com:5984/')
+    >>> secure_remote_server = Server('https://username:password@example.com:5984/')
 
     This class behaves like a dictionary of databases. For example, to get a
     list of database names on the server, you can simply iterate over the
@@ -225,6 +228,76 @@ class Server(object):
         status, headers, data = self.resource.post_json('_replicate', data)
         return data
 
+    def add_user(self, name, password, roles=None):
+        """Add regular user in authentication database.
+        :param name: name of regular user, normally user id
+        :param password: password of regular user
+        :param roles: roles of regular user
+        :return: (id, rev) tuple of the registered user
+        :rtype: `tuple`
+        """
+        user_db = self['_users']
+        return user_db.save({
+            '_id': 'org.couchdb.user:' + name,
+            'name': name,
+            'password': password,
+            'roles': roles or [],
+            'type': 'user',
+        })
+
+    def remove_user(self, name):
+        """Remove regular user in authentication database.
+        :param name: name of regular user, normally user id
+        """
+        user_db = self['_users']
+        doc_id = 'org.couchdb.user:' + name
+        del user_db[doc_id]
+
+    def login(self, name, password):
+        """Login regular user in couch db
+        :param name: name of regular user, normally user id
+        :param password: password of regular user
+        :return: authentication token
+        """
+        status, headers, _ = self.resource.post_json('_session', {
+            'name': name,
+            'password': password,
+        })
+        if sys.version_info[0] > 2:
+            cookie = headers._headers[0][1]
+        else:
+            cookie = headers.headers[0].split(';')[0]
+        pos = cookie.find('=')
+        return cookie[pos + 1:]
+
+    def logout(self, token):
+        """Logout regular user in couch db
+        :param token: token of login user
+        :return: True if successfully logout
+        :rtype: bool
+        """
+        header = {
+            'Accept': 'application/json',
+            'Cookie': 'AuthSession=' + token,
+        }
+        status, _, _ = self.resource.delete_json('_session', headers=header)
+        return status == 200
+
+    def verify_token(self, token):
+        """Verify user token
+        :param token: authentication token
+        :return: True if authenticated ok
+        :rtype: bool
+        """
+        try:
+            status, _, _ = self.resource.get_json('_session', {
+                'Accept': 'application/json',
+                'Cookie': 'AuthSession=' + token,
+            })
+        except http.Unauthorized:
+            return False
+        return status == 200
+
 
 class Database(object):
     """Representation of a database on a CouchDB server.
@@ -271,6 +344,12 @@ class Database(object):
     2
 
     >>> del server['python-tests']
+
+    If you need to connect to a database with an unverified or self-signed SSL
+    certificate, you can re-initialize your ConnectionPool as follows (only
+    applicable for Python 2.7.9+):
+
+    >>> db.resource.session.disable_ssl_verification()
     """
 
     def __init__(self, url, name=None, session=None):
@@ -402,7 +481,7 @@ class Database(object):
 
         If doc has no _id then the server will allocate a random ID and a new
         document will be created. Otherwise the doc's _id will be used to
-        identity the document to create or update. Trying to update an existing
+        identify the document to create or update. Trying to update an existing
         document with an incorrect _rev will raise a ResourceConflict exception.
 
         Note that it is generally better to avoid saving documents with no _id
@@ -927,7 +1006,10 @@ class Database(object):
         :param name: the name of the update handler function in the format
                      ``designdoc/updatename``.
         :param docid: optional ID of a document to pass to the update handler.
-        :param options: optional query string parameters.
+        :param options: additional (optional) params to pass to the underlying
+                        http resource handler, including ``headers``, ``body``,
+                        and ```path```. Other arguments will be treated as
+                        query string params. See :class:`couchdb.http.Resource`
         :return: (headers, body) tuple, where headers is a dict of headers
                  returned from the list function and body is a readable
                  file-like instance
@@ -1003,7 +1085,8 @@ class Document(dict):
 
         :rtype: basestring
         """
-        return self['_id']
+        return self.get('_id')
+
 
     @property
     def rev(self):
@@ -1011,7 +1094,7 @@ class Document(dict):
 
         :rtype: basestring
         """
-        return self['_rev']
+        return self.get('_rev')
 
 
 class View(object):
