@@ -17,6 +17,7 @@ import errno
 import socket
 import time
 import sys
+import ssl
 
 try:
     from threading import Lock
@@ -53,7 +54,7 @@ if sys.version < '2.7':
 
         Based on code originally copied from Python 2.7's httplib module.
         """
-        
+
         def endheaders(self, message_body=None):
             if self.__dict__['_HTTPConnection__state'] == _CS_REQ_STARTED:
                 self.__dict__['_HTTPConnection__state'] = _CS_REQ_SENT
@@ -159,24 +160,39 @@ class ResponseBody(object):
 
     def close(self):
         while not self.resp.isclosed():
-            self.resp.read(CHUNK_SIZE)
+            chunk = self.resp.read(CHUNK_SIZE)
+            if not chunk:
+                self.resp.close()
         if self.conn:
             self._release_conn()
 
     def iterchunks(self):
         assert self.chunked
+        buffer = []
         while True:
+
             if self.resp.isclosed():
                 break
+
             chunksz = int(self.resp.fp.readline().strip(), 16)
             if not chunksz:
                 self.resp.fp.read(2) #crlf
                 self.resp.close()
                 self._release_conn()
                 break
+
             chunk = self.resp.fp.read(chunksz)
-            for ln in chunk.splitlines():
-                yield ln
+            for ln in chunk.splitlines(True):
+
+                end = ln == b'\n' and not buffer # end of response
+                if not ln or end:
+                    break
+
+                buffer.append(ln)
+                if ln.endswith(b'\n'):
+                    yield b''.join(buffer)
+                    buffer = []
+
             self.resp.fp.read(2) #crlf
 
 
@@ -216,10 +232,24 @@ class Session(object):
         self.cache = cache
         self.max_redirects = max_redirects
         self.perm_redirects = {}
-        self.connection_pool = ConnectionPool(timeout)
+
+        self._disable_ssl_verification = False
+        self._timeout = timeout
+        self.connection_pool = ConnectionPool(
+            self._timeout,
+            disable_ssl_verification=self._disable_ssl_verification)
+
         self.retry_delays = list(retry_delays) # We don't want this changing on us.
         self.retryable_errors = set(retryable_errors)
 
+    def disable_ssl_verification(self):
+        """Disable verification of SSL certificates and re-initialize the
+        ConnectionPool. Only applicable on Python 2.7.9+ as previous versions
+        of Python don't verify SSL certs."""
+        self._disable_ssl_verification = True
+        self.connection_pool = ConnectionPool(self._timeout,
+            disable_ssl_verification=self._disable_ssl_verification)
+
     def request(self, method, url, body=None, headers=None, credentials=None,
                 num_redirects=0):
         if url in self.perm_redirects:
@@ -332,6 +362,14 @@ class Session(object):
             if num_redirects > self.max_redirects:
                 raise RedirectLimit('Redirection limit exceeded')
             location = resp.getheader('location')
+            
+            # in case of relative location: add scheme and host to the location
+            location_split = util.urlsplit(location)
+
+            if not location_split[0]:
+                orig_url_split = util.urlsplit(url)
+                location = util.urlunsplit(orig_url_split[:2] + location_split[2:])
+
             if status == 301:
                 self.perm_redirects[url] = location
             elif status == 303:
@@ -420,11 +458,22 @@ class Cache(object):
         self.by_url = dict(ls[-self.keep_size:])
 
 
+class InsecureHTTPSConnection(HTTPSConnection):
+    """Wrapper class to create an HTTPSConnection without SSL verification
+    (the default behavior in Python < 2.7.9). See:
+    https://docs.python.org/2/library/httplib.html#httplib.HTTPSConnection"""
+    if sys.version_info >= (2, 7, 9):
+        def __init__(self, *a, **k):
+            k['context'] = ssl._create_unverified_context()
+            HTTPSConnection.__init__(self, *a, **k)
+
+
 class ConnectionPool(object):
     """HTTP connection pool."""
 
-    def __init__(self, timeout):
+    def __init__(self, timeout, disable_ssl_verification=False):
         self.timeout = timeout
+        self.disable_ssl_verification = disable_ssl_verification
         self.conns = {} # HTTP connections keyed by (scheme, host)
         self.lock = Lock()
 
@@ -448,7 +497,10 @@ class ConnectionPool(object):
             if scheme == 'http':
                 cls = HTTPConnection
             elif scheme == 'https':
-                cls = HTTPSConnection
+                if self.disable_ssl_verification:
+                    cls = InsecureHTTPSConnection
+                else:
+                    cls = HTTPSConnection
             else:
                 raise ValueError('%s is not a supported scheme' % scheme)
             conn = cls(host, timeout=self.timeout)
